#include <string>

using namespace std;

// This class is a function wrapper that does partial function
// application. It stores the first argument and provides a call
// operator that is able to call functions with three arguments,
// to which the first argument is automatically passed.
//
// To keep things simple for demonstration purposes, the actual
// function is not stored in the Binder object and has to be passed
// to the call operator.
template<class FirstArg>
class Binder
{
public:
    Binder(FirstArg firstArgument)
        : m_firstArgument(firstArgument)
    {}

    //
    // The two arguments passed to the call operator are forwarded
    // to the called function. For this, we want to have perfect
    // forwarding - lvalues should  be forwarded as lvalues, and
    // rvalues should be forwarded as rvalues.
    //
    // To achieve this, we can simply use universal references and
    // std::forward().
    //

    template<class Function, class A, class B>
    void operator() (Function func,
                     A&& secondArgument, B&& thirdArgument)
    {
        func(m_firstArgument, forward<A>(secondArgument),
             forward<B>(thirdArgument));
    }

private:
    FirstArg m_firstArgument;
};

void baz(int, const string &, const string &) {}
void foo(int, const string &, string &&) {}
void bar(int, string &&, string &&) {}

string rvalueString() { return "Bonjour!"; }
static const string lvalueString = "Hello!";

int main()
{
    Binder<int> binder{42};

    binder(baz, lvalueString, lvalueString);
    binder(foo, lvalueString, rvalueString());
    binder(bar, rvalueString(), rvalueString());

    return 0;
}
