#include <iostream>
#include <algorithm>
#include <functional>
#include <cmath>
#include <vector>
#include <stdexcept>

using namespace std;
using namespace std::placeholders;

int sum(int a, int b, int c)
{
    return a + b + c;
}

int weirdAdd(int a, function<int(int)> func)
{
    return a + func(a);
}

// Should be using a range (two iterators) instead of a container, but
// that was simplified for brevity
template<typename Container>
void reverse_for_each(const Container &cont,
                      function<void(typename Container::value_type)> func)
{
    auto it = cont.crbegin();
    while (it != cont.crend()) {
        func(*it);
        ++it;
    }
}

enum MathType {Sum, Product};
function<int(int)> mathFunc(MathType type, int argument1)
{
    if (type == Product) {
        return [=](int input) {
            return input * argument1;
        };
    } else if (type == Sum) {
        return [=](int input) {
            return input + argument1;
        };
    } else throw runtime_error("Wrong type!");
}

int main()
{
    auto sum_alias = bind(sum, _1, _2, _3);
    cout << sum_alias(2, 4, 5) << endl; // Prints 11
    cout << endl;

    auto sum20 = bind(sum, 20, _1, _2);
    cout << sum20(2, 4) << endl; // Prints 26
    cout << endl;

    auto triple = bind(sum, _1, _1, _1);
    cout << triple(50) << endl; // Prints 150
    cout << endl;

    cout << weirdAdd(5, triple) << endl; // Prints 20
    cout << endl;

    auto pow7 = bind(pow<int, int>, _1, 7);
    vector<int> vec{1, 2, 3, 4};
    transform(vec.begin(), vec.end(), vec.begin(),
              pow7);
    for (int i : vec)
        cout << i << " " << endl;
    cout << endl;

    // Auto type deduction leads to template argument deduction for the
    // algorithm here, therefore use function instead of auto.
    function<void(int)> printer = [](int i) {
        cout << i << endl;
    };
    reverse_for_each(vec, printer);
    cout << endl;

    auto add12 = mathFunc(Sum, 12);
    cout << add12(9) << endl; // Prints 21
    cout << endl;
}
