#include <memory>
#include <iostream>

class Tracer
{
public:
    Tracer()
    {
        m_instanceNum = ++s_instanceCount;
        std::cout << "Tracer " << m_instanceNum << " created" << std::endl;
    }

    ~Tracer()
    {
        std::cout << "Tracer " << m_instanceNum << " destroyed" << std::endl;
    }

    Tracer(const Tracer &other)
    {
        m_instanceNum = ++s_instanceCount;
        std::cout << "Tracer " << other.m_instanceNum << " copied" << std::endl;
    }

    Tracer(Tracer &&other)
    {
        m_instanceNum = ++s_instanceCount;
        std::cout << "Tracer " << other.m_instanceNum << " moved" << std::endl;
    }

    void ping() const
    {
        std::cout << "Tracer " << m_instanceNum << " pinged" << std::endl;
    }

    int m_instanceNum;
    static int s_instanceCount;
};

int Tracer::s_instanceCount = 0;

class Widget
{
public:
    Widget(int data)
        : m_data(data)
    {}

    int data() const
    {
        return m_data;
    }

private:
    int m_data;
};

int longCalculation(int input)
{
    std::cout << "Running calculation..." << std::endl;
    return input * 2;
}

auto createWidget(int input)
{
    return std::make_unique<Widget>(input);
}


auto createRunnable(int input)
{
    Tracer tracer;
    std::unique_ptr<Widget> widget = createWidget(input);
    return [w = std::move(widget), tracer] {
        tracer.ping();
        return longCalculation(w->data());
    };
}

int main(int , char *[])
{
    {
        auto runnable = createRunnable(23);
        const int result = runnable();
        std::cout << "Result is: " << result << std::endl;

        // lambda has captured a std::unique_ptr, a move-only class, as member, and is therefore move-only itself
        //auto runnableCopy = runnable;
    }
    std::cout << "Returning to OS." << std::endl;
    return 0;
}
