#include <algorithm>
#include <iostream>

#include <cassert>

static const struct {      // unnamed type
    int myCode, theirCode;
} codeMap[] = {
    { 0,  1 },
    { 1,  2 },
    { 2, 12 },
    // ...
};

static int myCodeToTheirCode(int myCode) {
    struct SortByMyCode { // local type
        typedef bool result_type;
        typedef decltype(codeMap[0]) arg_type; // give the unnamed type a name, so we can reference it

        result_type operator()(arg_type & lhs, arg_type & rhs) const
        { return lhs.myCode < rhs.myCode; }

        result_type operator()(int lhs, arg_type & rhs) const
        { return lhs < rhs.myCode; }

        result_type operator()(arg_type & lhs, int rhs) const
        { return lhs.myCode < rhs; }
    };

    // using both local and unnamed types as template arguments:
    assert( std::is_sorted( std::begin(codeMap), std::end(codeMap),
                            SortByMyCode() ) );
    const auto it = std::lower_bound( std::begin(codeMap), std::end(codeMap),
                                      myCode, SortByMyCode() );
    if ( it == std::end(codeMap) || it->myCode != myCode )
        return -1; // not found
    else
        return it->theirCode;
}

int main() {
    std::cout << 0 << " -> " << myCodeToTheirCode(0) << std::endl;
    std::cout << 1 << " -> " << myCodeToTheirCode(1) << std::endl;
    std::cout << 2 << " -> " << myCodeToTheirCode(2) << std::endl;
}
