#ifdef USE_BOOST
#  include <boost/regex.hpp>
#else
#  include <regex>
#endif

#include <iostream>

using namespace std;
#ifdef USE_BOOST
using namespace boost;
#endif

int main()
{
    string files{"main.cpp background.jpg README.txt icon.png anim.gif"};

    // Here the .* at the start and beginning are required, as regex_match
    // matches the complete string, as opposed to regex_search
    regex txtFile{".*.txt.*"};
    cout << boolalpha << "List contains a text file: "
         << regex_match(files, txtFile) << endl;

    regex txtFile2{R"((\w+).txt)"};
    smatch txtMatch;
    regex_search(files, txtMatch, txtFile2);
    if (!txtMatch.empty())
        cout << R"(List contains a text file with the basename ")"
             << txtMatch[1] << R"(")" << endl;

    regex imageEx{R"((\w+).(jpg|png|gif))", regex_constants::icase};

    cout << endl << "Iterating over all images:" << endl;
    sregex_iterator it{begin(files), end(files), imageEx};
    while (it != sregex_iterator{}) {
        const smatch &match = *it;
        cout << "Image: " << match[0] << endl;
        cout << "   Basename: " << match[1] << endl;
        cout << "   Type: " << match[2] << endl;
        ++it;
    }

    // This actually iterates over all non-matches parts, so it includes the
    // whitespace.
    cout << endl << "Iterating over all non-images:" << endl;
    sregex_token_iterator it2{begin(files), end(files), imageEx, -1};
    while (it2 != sregex_token_iterator{}) {
        const ssub_match subMatch = *it2;
        cout << "Non-Image: " << subMatch << endl;
        ++it2;
    }

    cout << endl << "Iterating over all image types:" << endl;
    sregex_token_iterator it3{begin(files), end(files), imageEx, 2};
    while (it3 != sregex_token_iterator{}) {
        const ssub_match subMatch = *it3;
        cout << "Image type: " << subMatch << endl;
        ++it3;
    }

    cout << endl << "Renaming all image files to .bmp: ";
    cout << regex_replace(files, imageEx, string{"$1.bmp"}) << endl;
}

