#include <memory>
#include <iostream>
#include <string>

class Data {
public:
    ~Data() {
        std::cout << "I'm being deleted!" << std::endl;
    }
    void echo(const std::string &context) {
        std::cout << context << ": " << this << std::endl;
    }
};


/// A function which allocates a Data pointer and returns it.
/// The ownership of the pointer is handled by std::shared_ptr.
std::shared_ptr<Data> makeData()
{
    // Pattern:
    // 1) Always one per line, always directly into the shared pointer
    // 2) Using make_shared() (see below)
    std::shared_ptr<Data> result{new Data};

    // Alternative below, but we don't use the auto keyword here to make it
    // clearer where the shared_ptrs are used.
    //auto result = std::make_shared<Data>();

    result->echo("within makeData");

    return result;
}

void useData()
{
    std::shared_ptr<Data> data = makeData();
    data->echo("returned from makeData");
    std::cout << "the pointer held is: " << data.get() << std::endl;

    // Just for show off:
    std::shared_ptr<Data> data2 = data;
    data->echo("data after copy");
    data2->echo("data2 after copy");

    // The actual Data object gets deleted when the last shared pointer pointing to
    // it goes out of scope, i.e. when both data and data2 get destroyed.
}

int main(int, char *[])
{
    useData();
    return 0;
}
