#include <iostream>
#include <vector>
#include <string>
#include <functional>
#include <type_traits>
#include <chrono>

using namespace std;

// There already is a typedef for chrono::milliseconds, however that one
// uses integer representation, whereas we need floating point representation
// for more accuracy.
using ms = chrono::duration<double, milli>;

class Trivial {
public:
    int a;
    int *b;
};

// Silly example: In this class, we want to make sure b always points
// to the a of the same object, therefore the default copy constructor
// would not suffice. Adding a custom copy constructor makes the class
// non-trivial.
// NOTE: Actual implementation commented out for benchmark fairness.
class NonTrivial {
public:
    NonTrivial()// :
        //a{42},
        //b{&a}
    {}

    NonTrivial(const NonTrivial &/*other*/) //:
        //a{other.a},
        //b{&a}
    {}

    int a;
    int *b;
};

static_assert(is_trivial<Trivial>::value, "Trivial not trivial!");
static_assert(!is_trivial<NonTrivial>::value, "NonTrivial trivial!");

// Not yet implemented in GCC 4.7.2
//static_assert(!is_trivially_copy_constructible<NonTrivial>::value, "NonTrivial trivially copyable!");

static void executeTest(std::function<void()> test, const string &name)
{
    cout << endl << endl;
    cout << "--------- Executing " << name << " -------------------" << endl;
    const auto start = chrono::high_resolution_clock::now();
    for (int i = 0; i < 100; i++)
        test();
    const auto elapsed = chrono::high_resolution_clock::now() - start;
    cout << name << " took" << ": " << ms{elapsed}.count() << " ms\n";
}

template<class T>
void test()
{
    T a[100000];
    T b[100000];
    std::copy(begin(a), end(a), begin(b));
}

int main()
{
    executeTest(test<Trivial>, "Trivial Type");
    executeTest(test<NonTrivial>, "Nontrivial Type");
}
