#include <iostream>
#include <string>
#include <unordered_map>

using namespace std;

struct Employee
{
    int id;
    string name;
    int salary;

    bool operator == (const Employee &other) const {
        return id == other.id;
    }
};

ostream& operator<< (ostream& stream, const Employee& employee)
{
    stream << "Employee " << employee.id << " (" << employee.name << "," << employee.salary << ")";
    return stream;
}

namespace std {
template<>
struct hash<Employee>
{
    size_t operator()(const Employee &employee) const
    {
        return hash<int>{}(employee.id);
    }
};
}

template<class KeyType>
struct bad_hash
{
    size_t operator()(const KeyType &) const {
        return 42;
    }
};

using Project = string;

template<class HashMap>
void printHashMap(const HashMap &map)
{
    cout << "Size: " << map.size() << endl;
    cout << "Load factor (average number of elements per bucket): " << map.load_factor() << endl;
    cout << "Max load factor: " << map.max_load_factor() << endl;
    cout << "Buckets: " << map.bucket_count() << endl;

    cout << endl << "Complete contents:" << endl;
    for (const auto &it : map) {
        cout << it.first << ": " << it.second << endl;
    }

    cout << endl << "Buckets:" << endl;
    for (size_t bucketNum = 0; bucketNum < map.bucket_count(); bucketNum++) {
        cout << "Bucket " << bucketNum << endl;
        auto bucketIt = map.cbegin(bucketNum);
        while (bucketIt != map.cend(bucketNum)) {
            cout << "    " << bucketIt->first << ": " << bucketIt->second;
            cout << " (hash: " << typename HashMap::hasher{}(bucketIt->first) << ")" << endl;
            ++bucketIt;
        }
    }
}

int main()
{
    // Uncomment this to see how a bad hashing function can degrade the performance
    // of unordered_map: All elements will land in the same bucket
    //unordered_map<string, int, bad_hash<string>> countryMap = {

    unordered_map<string, int> countryMap = {
        {"China", 1354},
        {"India", 1210},
        {"United States", 315},
        {"Indonesia", 237},
        {"Brazil", 193},
        {"Pakistan", 183},
        {"Nigeria", 170},
        {"Bangladesh", 152},
        {"Russia", 143},
        {"Japan", 127},
        {"Mexico", 112},
        {"Philippines", 92},
        {"Vietnam", 88},
        {"Ethiopia", 86},
        {"Egypt", 83},
        {"Germany", 82}
    };

    printHashMap(countryMap);

    unordered_map<Employee, Project> projectMap;
    Employee peter{0, "Peter Gibbons", 70000};
    projectMap.emplace(peter, Project{"Fixing Printer"});
    projectMap.emplace(Employee{1, "Michael Bolton", 50000}, Project{"Laundering Money"});
    projectMap.emplace(Employee{11, "Samir Nagheenanajar", 60000}, Project{"Creating TPS Reports"});

    printHashMap(projectMap);
    cout << "Peter's project: " << projectMap.at(peter) << endl;

    return 0;
}
