#include <iostream>
#include <vector>
#include <string>
#include <algorithm>

using namespace std;

//
// For brevity, everything was made public in the two classes.
//

class Person
{
public:
    string name;

    explicit Person(const string &name)
        : name(name)
    {
    }

    virtual ~Person() = default;

    virtual void printInfo() const {
        cout << name << endl;
    }
};

class Employee : public Person
{
public:
    enum Job {
        Sysadmin,
        SoftwareEngineer,
        Manager
    };

    Employee(const string &name, Job job, int salary)
        : Person(name), job(job), salary(salary), boss(0)
    {
        id = highestEmployeeId++;
    }

    Job job;
    int salary;
    Employee *boss;
    int id;
    static int highestEmployeeId;

    void prnitInfo() const {
        Person::printInfo();
        cout << "Id: " << id << ", Salary: " << salary << endl << endl;
    }
};

int Employee::highestEmployeeId = 1;

struct IsItEmployee
{
    bool operator() (const Employee &employee) {
        return employee.job == Employee::SoftwareEngineer ||
               employee.job == Employee::Sysadmin;
    }
};

int main()
{
    vector<Employee> employees;

    employees.push_back(Employee("Frank \"Hitman\" Hennessy",
                                 Employee::Sysadmin, 40000));
    employees.push_back(Employee("Earl \"Magic\" Walker",
                                 Employee::SoftwareEngineer, 45000));
    employees.push_back(Employee("Walter \"Heisenberg\" White",
                                 Employee::Manager, 50000));
    employees.push_back(Employee("Peter \"Wolf\" Sanderson",
                                 Employee::SoftwareEngineer, 39000));

    vector<Employee>::const_iterator it = employees.cbegin();
    for (; it != employees.cend(); ++it)
        it->printInfo();

    // Calculate how many people work in IT
    const auto itCount = count_if(employees.cbegin(), employees.cend(),
                                  IsItEmployee());
    cout << "A total of " << itCount << " people work in IT." << endl;

    // Find highest earner and print his name
    // TODO: Use an algorithm to simplify this,
    vector<Employee>::iterator highestEarner = employees.end();
    vector<Employee>::iterator it2 = employees.begin();
    while (it2 != employees.end()) {
        const Employee e = *it2++;
        if (highestEarner == employees.end() ||
            e.salary > highestEarner->salary)
            highestEarner = it2;
    }
    cout << "Highest earner in the company is " << highestEarner->name << endl << endl;

    // Now fire him because he is costing the company so much!
    employees.erase(highestEarner);

    // Give all other employees a raise of 10%!
    vector<Employee> newEmployees = employees;
    vector<Employee>::iterator it3 = newEmployees.begin();
    for (; it3 != newEmployees.end(); ++it3)
        it3->salary = static_cast<int>(it3->salary * 1.1f);

    // Try using for_each here!
    cout << "Old Salaries:" << endl;
    vector<Employee>::const_iterator it4 = employees.cbegin();
    for (; it4 != employees.cend(); ++it4)
        it4->printInfo();
    cout << endl << "New Salaries:" << endl;
    vector<Employee>::const_iterator it5 = newEmployees.cbegin();
    for (; it5 != employees.cend(); ++it5)
        it5->printInfo();

}
