#include <iostream>
#include <algorithm>
#include <vector>
#include <string>
#include <stdexcept>
#include <functional>
#include <type_traits>
#include <utility>
#include <chrono>

using namespace std;

// There already is a typedef for chrono::milliseconds, however that one
// uses integer representation, whereas we need floating point representation
// for more accuracy.
using ms = chrono::duration<double, milli>;

static int numCopyCtorsCalled = 0;

class BigObject
{
    enum { DataSize = 100000 };
public:

    BigObject()
        : data{new int[DataSize]}
    {
        //cout << "BigObject constructor" << endl;
        std::generate(data, data + DataSize, rand);
    }

    ~BigObject()
    {
        delete[] data;
        data = nullptr;
    }

    BigObject(const BigObject &other)
        : data{new int[DataSize]}
    {
        //cout << "BigObject copy constructor" << endl;
        numCopyCtorsCalled++;
        copy(other.data, other.data + DataSize, data);
    }

    void changeSomething()
    {
        data[rand() % DataSize] = rand();
    }

    void printContent() const
    {
        for_each(data, data + DataSize, [](int /*value*/) {
            //cout << value << ",";
        });
    }

private:
    int *data;
};

static BigObject createBigObject()
{
    BigObject foo;
    BigObject bar;

    // Two return statements to prevent NRVO optimization
    if (rand() == 0)
        return foo;
    return bar;
}

static BigObject createAnotherBigObject()
{
    const BigObject foo;
    const BigObject bar;

    if (rand() == 0)
        return foo;
    return bar;
}

static const BigObject createYetAnotherObject()
{
    return BigObject{};
}

static void workWithBigObject(const BigObject &object)
{
    BigObject copy{object};
    copy.changeSomething();
    copy.printContent();
}

static void doVectorStuff()
{
    vector<BigObject> bigObjects;
    for (int i = 0; i < 5; i++) {
        bigObjects.push_back({});
    }
}

static void testObjectReturn()
{
    createBigObject();
}

static void testObjectReturn2()
{
    createAnotherBigObject();
}

static void useTempObjectInFunction()
{
    BigObject temp;
    workWithBigObject(temp);
}

static void vectorResize()
{
    doVectorStuff();
}

static void useTempObjectInFunction2()
{
    workWithBigObject(createYetAnotherObject());
}

static void executeTest(std::function<void()> test, const string &name)
{
    cout << endl << endl;
    cout << "--------- Executing " << name << " -------------------" << endl;
    const int startCopyCtors = numCopyCtorsCalled;
    const auto start = chrono::high_resolution_clock::now();

    test();

    const auto elapsed = chrono::high_resolution_clock::now() - start;
    cout << name << " took" << ": " << ms{elapsed}.count() << " ms\n";
    cout << "Number of copy constructors called: "
         << numCopyCtorsCalled - startCopyCtors << endl;
}

int main()
{
    const auto start = chrono::high_resolution_clock::now();

    executeTest(testObjectReturn, "Object Return Test");
    executeTest(testObjectReturn2, "Object Return Test 2");
    executeTest(useTempObjectInFunction, "Using Object in Function Test");
    executeTest(vectorResize, "Resizing Vector Test");
    executeTest(useTempObjectInFunction2, "Using Object in Function Test 2");

    const auto elapsed = chrono::high_resolution_clock::now() - start;
    cout << endl << endl;
    cout << "All tests took" << ": " << ms{elapsed}.count() << " ms\n";
    cout << "Number of copy constructors called: " << numCopyCtorsCalled << endl;
}
