#include <algorithm>
#include <cassert>
#include <string>
#include <vector>
#include <type_traits>
#include <iterator>

using namespace std;

struct Circle
{
    explicit Circle(int radius)
        : radius{radius}
    {}

    int radius;
};

struct BoundingBox
{
    int width;
    int height;
};

BoundingBox calculateBoundingBox(const Circle &circle)
{
    return {circle.radius * 2, circle.radius * 2};
}

// Just a little wrapper around stoi() to have a function that takes exactly one argument.
int stringToInt(const string &input)
{
    return stoi(input, 0, 10);
}

template<class InputType, class Func>
using ResultVector = vector<decltype(  declval<Func>()( declval<InputType>() )  )>;

template<class InputType, class Func>
auto transformContainer(const vector<InputType> &input, Func func)
    -> ResultVector<InputType, Func> // This line not needed anymore in C++14
{
    using ReturnContainer = ResultVector<InputType, Func>;

    // Alternative, without using the "ResultVector" template above
    //using ReturnContainer = vector<decltype(  func( declval<InputType>() )  )>;

    // Another alternative is using result_of. result_of uses decltype and declval
    // internally, similar to what is done above.
    //using ReturnContainer = vector<typename result_of<Func(InputType)>::type>;

    ReturnContainer result;
    result.reserve(input.size());
    transform(begin(input), end(input), back_inserter(result), func);
    return result;
}

int main()
{
    vector<Circle> circles;
    circles.push_back(Circle{12});
    circles.push_back(Circle{42});

    vector<string> strings;
    strings.push_back("12");
    strings.push_back("42");
    strings.push_back("129");

    vector<BoundingBox> boundingBoxes = transformContainer(circles, calculateBoundingBox);
    assert(boundingBoxes.size() == circles.size());
    assert(boundingBoxes[0].width == circles[0].radius * 2);
    assert(boundingBoxes[0].height == circles[0].radius * 2);
    assert(boundingBoxes[1].width == circles[1].radius * 2);
    assert(boundingBoxes[1].height == circles[1].radius * 2);

    vector<int> numbers = transformContainer(strings, stringToInt);
    assert(numbers.size() == 3);
    assert(numbers[0] == 12);
    assert(numbers[1] == 42);
    assert(numbers[2] == 129);
}
