#include <iostream>
#include <thread>
#include <mutex>
#include <atomic>

using namespace std;

mutex m;
atomic_bool cancel{false};

bool isPrime(unsigned long long n)
{
    if (n <= 1)
        return false;

    for (unsigned long long i = 2; i < n; i++) {
        if (n % i == 0)
            return false;
    }

    return true;
}

unsigned long long fibonacci(unsigned long long n)
{
    if (n <= 2)
        return 1;
    else
        return fibonacci(n - 1) + fibonacci(n - 2);
}

void printPrime()
{
    unsigned long long highestPrime = 2;
    for (unsigned long long i = 2;; i++) {
        if (isPrime(i))
            highestPrime = i;

        const bool c = cancel.load(memory_order_acquire);
        if (c)
            break;
    }

    const lock_guard<mutex> guard{m};
    cout << "Highest prime found: " << highestPrime << endl;
}

void printFibonacci()
{
    const unsigned long long result = fibonacci(42);
    cancel.store(true, memory_order_release);

    const lock_guard<mutex> guard{m};
    cout << "Fibonacci of 42: " << result << endl;
}

int main()
{
    // This example keeps calculating prime numbers until the calculation
    // of fibonacci(42) has finished.
    thread t{printPrime};
    printFibonacci();

    // Join will wait until the printPrime thread has finished. Since the
    // cancel variable has been set to true, the printPrime thread will finish
    // soon, after it is done calculating the current iteration of isPrime().
    t.join();
}
