#include <iostream>
#include <thread>
#include <list>
#include <mutex>
#include <condition_variable>

using namespace std;

list<unsigned long long> primes;
mutex m;
condition_variable newPrimeCondition;

bool isPrime(unsigned long long n)
{
    if (n <= 1)
        return false;

    for (unsigned long long i = 2; i < n; i++) {
        if (n % i == 0)
            return false;
    }

    return true;
}

void findPrimes()
{
    // Very inefficient algorithm, but that is fine for demo purposes here
    for (unsigned long long i = 1;; i++) {
        if (isPrime(i)) {
            unique_lock<std::mutex> lock{m};
            primes.push_back(i);
            newPrimeCondition.notify_one();
        }
    }
}

int main()
{
    thread t{findPrimes};
    unique_lock<std::mutex> lock{m};

    for (;;) {
        newPrimeCondition.wait(lock, [] {
            return !primes.empty();
        });

        while (!primes.empty()) {
            cout << primes.front() << endl;
            primes.pop_front();
        }
    }
}
