#include <iostream>
#include <thread>
#include <vector>
#include <future>
#include <algorithm>
#include <numeric>

using namespace std;

bool isPrime(unsigned long long n)
{
    if (n <= 1)
        return false;

    for (unsigned long long i = 2; i < n; i++) {
        if (n % i == 0)
            return false;
    }

    return true;
}

unsigned long long nthPrime(unsigned long long n)
{
    unsigned long long numPrimesFound = 0;
    for (unsigned long long i = 1;; i++) {
        if (isPrime(i)) {
            numPrimesFound++;
            if (numPrimesFound == n)
                return i;
        }
    }
}

unsigned long long numberFromPrimeSummands()
{
    cout << "Creating array of futures..." << endl;

    // Creating the future will trigger starting the tasks in the background.
    // We can already store or pass around the futures even though the result is
    // not there yet.

    vector<future<unsigned long long>> primes(10);
    generate(primes.begin(), primes.end(), [](){
        return async(launch::async, []() {
            return nthPrime(rand() % 10000);
        });
    });

    cout << "Running accumulate algorithm..." << endl;

    // Actually accessing the future's result with get() will cause the get() to
    // block until the result is available

    return accumulate(primes.begin(), primes.end(), 0ull,
                      [](unsigned long long resultSoFar,
                         future<unsigned long long> &element) {
        return resultSoFar + element.get();
    });
}

int main()
{
    // Creating a number from prime factors would make more sense, but there, the
    // numbers would get too big quite quickly, and that wouldn't demonstrate long
    // operations as nicely.

    cout << "Number created from 10 prime summands: "
         << numberFromPrimeSummands() << endl;
}
