#include <iostream>
#include <future>
#include <vector>
#include <cstdint>

using namespace std;

// MSVC <= 2013 doesn't support generation of implicit move constructors, which we make
// use of (vector<task> contains the move-only type 'task')
#if defined(_MSC_VER) && (_MSC_VER <= 1800)
int main() {}
#else

// ------------------------------------------------------------------------------
// Some math helpers so we have functions that do some work
// ------------------------------------------------------------------------------
bool isPrime(uint64_t n)
{
    if (n <= 1)
        return false;

    for (uint64_t i = 2; i < n; i++) {
        if (n % i == 0)
            return false;
    }

    return true;
}

uint64_t nthPrime(uint64_t n)
{
    uint64_t numPrimesFound = 0;
    for (uint64_t i = 1;; i++) {
        if (isPrime(i)) {
            numPrimesFound++;
            if (numPrimesFound == n)
                return i;
        }
    }
}

uint64_t fibonacci(uint64_t n)
{
    if (n <= 2)
        return 1;
    else
        return fibonacci(n - 1) + fibonacci(n - 2);
}

uint64_t findPrime()
{
    return nthPrime(8000);
}

uint64_t findFibonacci()
{
    return fibonacci(43);
}

// ------------------------------------------------------------------------------
// Main part of the example
// ------------------------------------------------------------------------------

using task = packaged_task<uint64_t(void)>;

void executeTasks(vector<task> tasks)
{
    for (task &t : tasks) {
        cout << "Starting next task..." << endl;
        t();
        cout << "Task done." << endl;
    }
}

int main()
{
    task fibTask{findFibonacci};
    task primeTask{findPrime};

    future<uint64_t> fibResult = fibTask.get_future();
    future<uint64_t> primeResult = primeTask.get_future();

    vector<task> tasks;
    tasks.push_back(move(fibTask));
    tasks.push_back(move(primeTask));

    thread taskThread{executeTasks, move(tasks)};

    cout << "Some fibonacci number: " << fibResult.get() << endl;
    cout << "Some prime number: " << primeResult.get() << endl;

    taskThread.join();
}

#endif // _MSC_VER
