#include <chrono>
#include <iostream>
#include <vector>
#include <future>
#include <algorithm>

using namespace std;
using namespace std::chrono;

// thread_local support only available since MSVC 2014
#if (defined _MSC_VER) && (_MSC_VER <= 1800)
int main() {}
#else

struct RandomState
{
    RandomState()
        : next(static_cast<unsigned long>(high_resolution_clock::now().time_since_epoch().count()))
    {
        cout << "RandomState ctor of thread " << this_thread::get_id() << endl;
    }

    // Not yet supported by Clang on Linux, see https://llvm.org/bugs/show_bug.cgi?id=21738
#if (!defined __clang__) || (defined __APPLE__)
    ~RandomState()
    {
        cout << "RandomState dtor of thread " <<  this_thread::get_id() << endl;
    }
#endif

    unsigned long next;
};

thread_local RandomState randomState;

// thread-safe and fast "replacement" for rand()
static int myRand()
{
    randomState.next = randomState.next * 1103515245 + 12345;
    return (unsigned int)(randomState.next / 65536) % 32768;
}

vector<int> gatherRandomNumbers()
{
    vector<int> result(10000);
    generate(result.begin(), result.end(), myRand);
    return result;
}

int main()
{
    auto numbers1_f = async(launch::async, gatherRandomNumbers);
    auto numbers2_f = async(launch::async, gatherRandomNumbers);

    auto numbers1 = numbers1_f.get();
    auto numbers2 = numbers2_f.get();

    cout << "Thread 1\tThread 2" << endl;
    for (int i = 0; i < 10; i++) {
        int n1 = numbers1[i];
        int n2 = numbers2[i];
        cout << n1 << '\t' << n2 << endl;
    }
}

#endif // _MSC_VER
