#include <iostream>

//
//
// Length
//
//

class Length {
    long double m_meters;
public:
    constexpr explicit Length(long double meters=0)
        : m_meters{meters} {}

    // copy/move ctor/assignment operators are fine!

    constexpr long double meters() const { return m_meters; }
    
    void setMeters(long double meters) { m_meters = meters; }
};

// Equality:

constexpr bool operator==(const Length &lhs, const Length &rhs)
{ return lhs.meters() == rhs.meters(); }

constexpr bool operator!=(const Length &lhs, const Length &rhs)
{ return !operator==(lhs, rhs); }


// binary +:
constexpr Length operator+(const Length &lhs, const Length &rhs)
{ return Length{ lhs.meters() + rhs.meters() }; }


// binary -:
constexpr Length operator-(const Length &lhs, const Length &rhs)
{ return Length{ lhs.meters() - rhs.meters() }; }


// unary -:
constexpr Length operator-(const Length &len)
{ return Length{-len.meters()}; }


// enable (some) si units:

constexpr Length operator "" _cm(unsigned long long value)
{ return Length{value/100.0l}; }

constexpr Length operator "" _cm(long double value)
{ return Length{value/100.0}; }

constexpr Length operator "" _m(unsigned long long value)
{ return Length{static_cast<long double>(value)}; }

constexpr Length operator "" _m(long double value)
{ return Length{value}; }

constexpr Length operator "" _km(unsigned long long value)
{ return Length{static_cast<long double>(value*1000)}; }

constexpr Length operator "" _km(long double value)
{ return Length{value*1000}; }

//
//
// Area
//
//

class Area {
    long double m_squareMeters;
public:
    constexpr explicit Area(long double squareMeters=0)
        : m_squareMeters{squareMeters} {}

    constexpr long double squareMeters() const
    { return m_squareMeters; }

    void setSquareMeters(long double squareMeters)
    { m_squareMeters = squareMeters; }
};

// Equality:

constexpr bool operator==(const Area &lhs, const Area &rhs)
{ return lhs.squareMeters() == rhs.squareMeters(); }

constexpr bool operator!=(const Area &lhs, const Area &rhs)
{ return !operator==(lhs, rhs); }


// Length * Length:

constexpr Area operator*(const Length &lhs, const Length &rhs)
{ return Area{ lhs.meters() * rhs.meters() }; }

// enable (one) si unit:

constexpr Area operator "" _sqm(unsigned long long value)
{ return Area{static_cast<long double>(value)}; }

constexpr Area operator "" _sqm(long double value)
{ return Area{value}; }


// Area serialisation:

template <typename T_Char, typename T_Traits>
std::basic_ostream<T_Char,T_Traits>
&operator <<(std::basic_ostream<T_Char,T_Traits> &s, const Area &area)
{ return s << area.squareMeters() << "_sqm"; }


// test program:

int main() {

    constexpr Area area = -10._m * ( -5_m - 500.0_cm );

    static_assert(area == 100_sqm, "10m x 10m != 100Sq m???");

    std::cout << area << std::endl;

    return 0;
}
