#include <limits>
#include <cstdint>
#include <cassert>

// Uncomment to see compile-time errors for invalid binary numbers
// #define ERROR_CASE

template <char C, char... Cs>
constexpr unsigned long long binary_helper()
{
    static_assert( C == '1' || C == '0', "non-binary digit" );
    return ( binary_helper<C>() << sizeof...(Cs) ) | binary_helper<Cs...>() ;
}

template <>
constexpr unsigned long long binary_helper<'0'>()
{
    return 0;
}

template <>
constexpr unsigned long long binary_helper<'1'>()
{
    return 1;
}

template <char... Cs>
constexpr int operator "" _b()
{
	static_assert(sizeof...(Cs) <= 32, "binary literal too long for int");
	return static_cast<int>(binary_helper<Cs...>());
}

template <char... Cs>
constexpr unsigned int operator "" _bu()
{
	static_assert(sizeof...(Cs) <= 32, "binary litera; too long for unsigned int");
	return static_cast<unsigned int>(binary_helper<Cs...>());
}


static_assert( binary_helper<'1','0','1','0','1','0'>() == 42, "oops" );
static_assert(  101010_bu ==  42, "oops 2" );
static_assert( -101010_b  == -42, "oops 3" );
static_assert( 11111111111111111111111111111111_bu == std::numeric_limits<uint32_t>::max(), "oops 4" );
static_assert( 11111111111111111111111111111111_b  == -1, "oops 5" );

int main()
{
    assert(  101010_bu ==  42 );
    assert( -101010_b  == -42 );
    assert( 11111111111111111111111111111111_bu == std::numeric_limits<uint32_t>::max() );
    assert( 11111111111111111111111111111111_b  == -1 );

#ifdef ERROR_CASE
    /*const int error1 =*/ 2_b; // should error out ('2' isn't a binary digit)
    /*const int error2 =*/ 121_b; // should error out too
#endif

    return 0;
}

