#include <memory>
#include <iostream>

// This example shows how to implement a very basic std::function, which
// doesn't even support return types other than void.

template<typename ... Args>
class Callable
{
public:
    virtual void invoke(Args &&...args) = 0;
};

template<typename Func, typename ... Args>
class FunctorCallable : public Callable<Args...>
{
private:
    Func func;

public:
    FunctorCallable(Func functor)
        : func(functor)
    {}

    void invoke(Args &&...args) override
    {
        func(std::forward<Args>(args)...);
    }
};

template<typename ...Args>
class Function
{
private:
    std::shared_ptr<Callable<Args...>> m_callable;

public:
    template<typename Func>
    Function(Func func)
        : m_callable(std::make_shared<FunctorCallable<Func, Args...>>(func))
    {
    }

    void operator()(Args &&... args)
    {
        m_callable->invoke(std::forward<Args>(args)...);
    }
};





void add(int a, int b, int c)
{
    std::cout << a + b + c << std::endl;
}

void sayHello()
{
    std::cout << "Hello" << std::endl;
}

int main()
{
    Function<> f1{sayHello};
    f1();


    Function<int, int, int> f2{add};
    f2(7, 3, 5);


    int x = 100;
    auto multiply = [x](float y) {
        std::cout << x * y << std::endl;
    };
    Function<float> f3{multiply};
    f3(10);


    auto f4 = f3;
    f4(42);


    return 0;
}
