#include <tuple>
#include <iostream>

// Note: For simplicity's sake, perfect forwarding was not used here.

/*
 * C++11: Recursive template instantiations
 */

// Recursion step
template<int N, typename ...T>
struct tuple_helper_11
{
    template<typename Fun, typename ...U>
    static auto apply(const Fun &fun, const std::tuple<T...> &tuple,
                      U ... unpacked_elements)
    {
        return tuple_helper_11<N-1, T...>::apply(fun, tuple,
                                                 std::get<N-1>(tuple),
                                                 unpacked_elements...);
    }
};

// Recursion end
template<typename ...T>
struct tuple_helper_11<0, T...>
{
    template<typename Fun, typename ...U>
    static auto apply(const Fun &fun, const std::tuple<T...> &/*tuple*/,
                      U ... unpacked_elements)
    {
        return fun(unpacked_elements...);
    }
};

template<typename Fun, typename ...T>
auto apply_tuple_11(const Fun &fun, const std::tuple<T...> &tuple)
{
    return tuple_helper_11<sizeof...(T), T...>::apply(fun, tuple);
}

/*
 * C++14: index_sequence, no recursion
 */

template<typename Fun, typename Tuple, size_t ...Indices>
auto tuple_helper_14(const Fun &fun, const Tuple& tuple, std::index_sequence<Indices...> /* indices */)
{
    return fun(std::get<Indices>(tuple)...);
}

template<typename Fun, typename Tuple>
auto apply_tuple_14(const Fun &fun, const Tuple &tuple)
{
    std::make_index_sequence<std::tuple_size<Tuple>::value> indices;
    return tuple_helper_14(fun, tuple, indices);
}

/*
 * Example of using both apply_tuple_11() and apply_tuple_14()
 */

void printSum(int a, float b, double c)
{
    std::cout << a + b + c << std::endl;
}

void printProduct(float a, double b, int c)
{
    std::cout << a * b * c << std::endl;
}

int main(int , char *[])
{
    auto tuple1 = std::make_tuple(2, 4.0f, 7.0);
	auto tuple2 = std::make_tuple(2.0f, 4.0, 7);

    apply_tuple_11(printSum, tuple1);
    apply_tuple_11(printProduct, tuple2);

    apply_tuple_14(printSum, tuple1);
    apply_tuple_14(printProduct, tuple2);
}
