#include <iostream>
#include <stdexcept>
#include <complex>

#include <cassert>

namespace safe {

    void printf(const char *fmt)
    {
        while (fmt && *fmt) {
            if (*fmt == '%' && *++fmt != '%')
                throw std::runtime_error("invalid format: surplus arguments");
            std::cout << *fmt;
            ++fmt;
        }
    }

    template <typename T, typename... Ts>
    void printf(const char *fmt, const T &value, const Ts &... ts)
    {
        if (!fmt)
            throw std::runtime_error("invalid format: arguments, but null format");
        while (*fmt) {
            if ( *fmt == '%' && *++fmt != '%' ) {
                if (!*fmt)
                    throw std::runtime_error("invalid format: ended in '%'");
                std::cout << value;
                return printf(++fmt, ts...);
            }
            std::cout << *fmt;
            ++fmt;
        }
        throw std::runtime_error("invalid format: too few placeholders");
    }

}

// not provided by standard?
template <typename D1, typename D2, typename R=decltype(D1{0}+D2{0})>
std::complex<R> operator+( std::complex<D1> lhs, std::complex<D2> rhs )
{
    return std::complex<R>{ lhs.real()+rhs.real(), lhs.imag()+rhs.imag() };
}

int main() {
    const std::complex<double> cd{1,1};
    const std::complex<float>  cf{2,2};
    const auto cs = cd + cf;

    safe::printf("Hello, World! 100%%\n");
    safe::printf("Hello, complex, 100%%: %#\n", cd);
    safe::printf("Hello, complexes, 100%%: %# + %# = %#\n", cd, cf, cs);

    return 0;
}
